from typing import Union

from tapo.device_management_ext import DeviceManagementExt
from tapo.requests import (
    Color,
    ColorLightSetDeviceInfoParams,
    LightingEffect,
    LightingEffectPreset,
    SegmentEffect,
    SegmentEffectPreset,
)
from tapo.responses import DeviceInfoRgbicLightStripResult, DeviceUsageResult

class RgbicLightStripHandler(DeviceManagementExt):
    """Handler for the [L920](https://www.tapo.com/en/search/?q=L920) and
    [L930](https://www.tapo.com/en/search/?q=L930) devices.
    """

    def __init__(self, handler: object):
        """Private constructor.
        It should not be called from outside the tapo library.
        """

    async def refresh_session(self) -> None:
        """Refreshes the authentication session."""

    async def on(self) -> None:
        """Turns *on* the device."""

    async def off(self) -> None:
        """Turns *off* the device."""

    async def get_device_info(self) -> DeviceInfoRgbicLightStripResult:
        """Returns *device info* as `DeviceInfoRgbicLightStripResult`.
        It is not guaranteed to contain all the properties returned from the Tapo API.
        If the deserialization fails, or if a property that you care about it's not present,
        try `RgbicLightStripHandler.get_device_info_json`.

        Returns:
            DeviceInfoRgbicLightStripResult: Device info of Tapo L920 and L930.
            Superset of `GenericDeviceInfoResult`.
        """

    async def get_device_info_json(self) -> dict:
        """Returns *device info* as json.
        It contains all the properties returned from the Tapo API.

        Returns:
            dict: Device info as a dictionary.
        """

    async def get_device_usage(self) -> DeviceUsageResult:
        """Returns *device usage* as `DeviceUsageResult`.

        Returns:
            DeviceUsageResult: Contains the time usage.
        """

    def set(self) -> ColorLightSetDeviceInfoParams:
        """Returns a `ColorLightSetDeviceInfoParams` builder that allows
        multiple properties to be set in a single request.
        `ColorLightSetDeviceInfoParams.send` must be called at the end to apply the changes.

        Returns:
            ColorLightSetDeviceInfoParams: Builder that is used by the
            `RgbicLightStripHandler.set` API to set multiple properties in a single request.
        """

    async def set_brightness(self, brightness: int) -> None:
        """Sets the *brightness* and turns *on* the device.

        Args:
            brightness (int): between 1 and 100
        """

    async def set_color(self, color: Color) -> None:
        """Sets the *color* and turns *on* the device.

        Args:
            color (Color): one of `tapo.Color` as defined in the Google Home app.
        """

    async def set_hue_saturation(self, hue: int, saturation: int) -> None:
        """Sets the *hue*, *saturation* and turns *on* the device.

        Args:
            hue (int): between 0 and 360
            saturation (int): between 1 and 100
        """

    async def set_color_temperature(self, color_temperature: int) -> None:
        """Sets the *color temperature* and turns *on* the device.

        Args:
            color_temperature (int): between 2500 and 6500
        """

    async def set_lighting_effect(
        self, lighting_effect: Union[LightingEffect, LightingEffectPreset]
    ) -> None:
        """Sets a *lighting effect* and turns *on* the device.

        Args:
            lighting_effect (LightingEffect | LightingEffectPreset)
        """

    async def set_segment_effect(
        self, segment_effect: Union[SegmentEffect, SegmentEffectPreset]
    ) -> None:
        """Sets a *segment effect* and turns *on* the device.

        This is used for the newer app-defined RGBIC strip effects that cannot be set by
        `LightingEffect` (for example, "circulating" or "breathe" segment effects).

        Args:
            segment_effect (SegmentEffect | SegmentEffectPreset)
        """



from enum import Enum
from typing import List, Optional, Tuple

class SegmentEffectType(str, Enum):
    Circulating = "Circulating"
    Breathe = "Breathe"
    Chasing = "Chasing"
    Flicker = "Flicker"
    Bloom = "Bloom"
    Stacking = "Stacking"

class SegmentEffect:
    brightness: int
    is_custom: bool
    device_type: Optional[str]
    display_colors: List[Tuple[int, int, int, int]]
    enabled: bool
    id: str
    name: str
    segments: Optional[List[int]]
    states: Optional[List[Tuple[int, int, int, int]]]
    type: Optional[str]

    def __init__(
        self,
        name: str,
        type: SegmentEffectType,
        is_custom: bool,
        enabled: bool,
        brightness: int,
        display_colors: List[Tuple[int, int, int, int]],
    ) -> None: ...
    def with_brightness(self, brightness: int) -> SegmentEffect: ...
    def with_is_custom(self, is_custom: bool) -> SegmentEffect: ...
    def with_display_colors(
        self, display_colors: List[Tuple[int, int, int, int]]
    ) -> SegmentEffect: ...
    def with_enabled(self, enabled: bool) -> SegmentEffect: ...
    def with_id(self, id: str) -> SegmentEffect: ...
    def with_name(self, name: str) -> SegmentEffect: ...
    def with_type(self, type: SegmentEffectType) -> SegmentEffect: ...
    def with_device_type(self, device_type: str) -> SegmentEffect: ...
    def with_segments(self, segments: List[int]) -> SegmentEffect: ...
    def with_states(self, states: List[Tuple[int, int, int, int]]) -> SegmentEffect: ...

class SegmentEffectPreset(str, Enum):
    Birthday = "Birthday"
    Blue = "Blue"
    Bonfire = "Bonfire"
    Candlelight = "Candlelight"
    Carnival = "Carnival"
    Cyan = "Cyan"
    Dancing = "Dancing"
    Dating = "Dating"
    Disco = "Disco"
    Dreamland = "Dreamland"
    ElectroDance = "ElectroDance"
    Energetic = "Energetic"
    Excited = "Excited"
    Fall = "Fall"
    Family = "Family"
    Fireworks = "Fireworks"
    FlowerField = "FlowerField"
    Forest = "Forest"
    Game = "Game"
    Green = "Green"
    Halloween = "Halloween"
    Happy = "Happy"
    Jazz = "Jazz"
    Lake = "Lake"
    LightGreen = "LightGreen"
    Lyric = "Lyric"
    Moonlight = "Moonlight"
    Morning = "Morning"
    Movie = "Movie"
    NewYear = "NewYear"
    Night = "Night"
    Orange = "Orange"
    Pink = "Pink"
    Purple = "Purple"
    Quiet = "Quiet"
    Red = "Red"
    Relaxed = "Relaxed"
    Rock = "Rock"
    Siren = "Siren"
    Sleep = "Sleep"
    Snow = "Snow"
    Star = "Star"
    Study = "Study"
    Summer = "Summer"
    Sunny = "Sunny"
    Sweet = "Sweet"
    Tense = "Tense"
    Thinking = "Thinking"
    Universe = "Universe"
    Volcano = "Volcano"
    Warm = "Warm"
    White = "White"
    Winter = "Winter"
    Work = "Work"
    Yellow = "Yellow"



from .energy_data_interval import *
from .play_alarm import *
from .power_data_interval import *
from .segment_effect_rule import *
from .set_device_info import *

from tapo.responses import TemperatureUnitKE100 as TemperatureUnitKE100



from enum import Enum

class PowerDataInterval(str, Enum):
    """Power data interval."""

    Every5Minutes = "5min"
    """Every 5 minutes interval. `start_date_time` and `end_date_time` describe an exclusive interval.
    If the result would yield more than 144 entries (i.e. 12 hours),
    the `end_date_time` will be adjusted to an earlier date and time.
    """

    Hourly = "Hourly"
    """Hourly interval. `start_date_time` and `end_date_time` describe an exclusive interval.
    If the result would yield more than 144 entries (i.e. 6 days),
    the `end_date_time` will be adjusted to an earlier date and time.
    """



"""Discover devices on the local network Example"""

import asyncio
import os

from tapo import ApiClient, DiscoveryResult


async def main():
    tapo_username = os.getenv("TAPO_USERNAME")
    tapo_password = os.getenv("TAPO_PASSWORD")
    target = os.getenv("TARGET", "192.168.1.255")
    timeout_s = int(os.getenv("TIMEOUT", 10))

    print(f"Discovering Tapo devices on target: {target} for {timeout_s} seconds...")

    api_client = ApiClient(tapo_username, tapo_password)
    discovery = await api_client.discover_devices(target, timeout_s)

    async for discovery_result in discovery:
        try:
            device = discovery_result.get()

            match device:
                case DiscoveryResult.GenericDevice(device_info, _handler):
                    print(
                        f"Found Unsupported Device '{device_info.nickname}' of model '{device_info.model}' at IP address '{device_info.ip}'."
                    )
                case DiscoveryResult.Light(device_info, _handler):
                    print(
                        f"Found '{device_info.nickname}' of model '{device_info.model}' at IP address '{device_info.ip}'."
                    )
                case DiscoveryResult.ColorLight(device_info, _handler):
                    print(
                        f"Found '{device_info.nickname}' of model '{device_info.model}' at IP address '{device_info.ip}'."
                    )
                case DiscoveryResult.RgbLightStrip(device_info, _handler):
                    print(
                        f"Found '{device_info.nickname}' of model '{device_info.model}' at IP address '{device_info.ip}'."
                    )
                case DiscoveryResult.RgbicLightStrip(device_info, _handler):
                    print(
                        f"Found '{device_info.nickname}' of model '{device_info.model}' at IP address '{device_info.ip}'."
                    )
                case DiscoveryResult.Plug(device_info, _handler):
                    print(
                        f"Found '{device_info.nickname}' of model '{device_info.model}' at IP address '{device_info.ip}'."
                    )
                case DiscoveryResult.PlugEnergyMonitoring(device_info, _handler):
                    print(
                        f"Found '{device_info.nickname}' of model '{device_info.model}' at IP address '{device_info.ip}'."
                    )
                case DiscoveryResult.PowerStrip(device_info, _handler):
                    print(
                        f"Found Power Strip of model '{device_info.model}' at IP address '{device_info.ip}'."
                    )
                case DiscoveryResult.PowerStripEnergyMonitoring(device_info, _handler):
                    print(
                        f"Found Power Strip with Energy Monitoring of model '{device_info.model}' at IP address '{device_info.ip}'."
                    )
                case DiscoveryResult.Hub(device_info, _handler):
                    print(
                        f"Found '{device_info.nickname}' of model '{device_info.model}' at IP address '{device_info.ip}'."
                    )
        except Exception as e:
            print(f"Error discovering device: {e}")


if __name__ == "__main__":
    asyncio.run(main())



class DeviceManagementExt:
    """Extension class for device management capabilities like `device_reboot` and `device_reset`."""

    async def device_reboot(self, delay_s: int) -> None:
        """*Reboots* the device.

        Notes:
            * Using a very small delay (e.g. 0 seconds) may cause a `ConnectionReset` or `TimedOut` error as the device reboots immediately.
            * Using a larger delay (e.g. 2-3 seconds) allows the device to respond before rebooting, reducing the chance of errors.
            * With larger delays, the method completes successfully before the device reboots.
              However, subsequent commands may fail if sent during the reboot process or before the device reconnects to the network.

        Args:
            delay_s (int): The delay in seconds before the device is rebooted.
        """

    async def device_reset(self) -> None:
        """*Hardware resets* the device.

        Warning:
            This action will reset the device to its factory settings.
            The connection to the Wi-Fi network and the Tapo app will be lost,
            and the device will need to be reconfigured.

        This feature is especially useful when the device is difficult to access
        and requires reconfiguration.
        """



from typing import List, Optional

from tapo.responses.device_info_result.default_state import DefaultStateType

class DeviceInfoRgbLightStripResult:
    """Device info of Tapo L900. Superset of `GenericDeviceInfoResult`."""

    device_id: str
    type: str
    model: str
    hw_id: str
    hw_ver: str
    fw_id: str
    fw_ver: str
    oem_id: str
    mac: str
    ip: str
    ssid: str
    signal_level: int
    rssi: int
    specs: str
    lang: str
    device_on: bool
    on_time: int
    """The time in seconds this device has been ON since the last state change (On/Off)."""
    nickname: str
    avatar: str
    has_set_location_info: bool
    region: Optional[str]
    latitude: Optional[float]
    longitude: Optional[float]
    time_diff: Optional[int]

    # Unique to this device
    brightness: int
    color_temp_range: List[int]
    color_temp: int
    default_states: DefaultRgbLightStripState
    """The default state of a device to be used when internet connectivity is lost after a power cut."""
    hue: Optional[int]
    overheated: bool
    saturation: Optional[int]

    def to_dict(self) -> dict:
        """Gets all the properties of this result as a dictionary.

        Returns:
            dict: The result as a dictionary.
        """

class DefaultRgbLightStripState:
    """RGB Light Strip Default State."""

    type: DefaultStateType
    state: RgbLightStripState

class RgbLightStripState:
    """RGB Light Strip State."""

    brightness: Optional[int]
    hue: Optional[int]
    saturation: Optional[int]
    color_temp: Optional[int]



from datetime import datetime
from typing import Optional, Tuple

class EnergyUsageResult:
    """Contains local time, current power and the energy usage and runtime for today and for the current month."""

    current_power: Optional[int]
    """Current power in milliwatts (mW)."""
    electricity_charge: Optional[Tuple[int, int, int]]
    """Electricity charge/cost data reported by the device using the tariff configured in the Tapo app.
    The third element is the total charge for the current month.
    The meaning of the first two elements is not confirmed; please open an issue or pull request if you know."""
    local_time: datetime
    """Local time of the device."""
    month_energy: int
    """Current month energy usage in Watt Hours (Wh)."""
    month_runtime: int
    """Current month runtime in minutes."""
    today_energy: int
    """Today energy usage in Watt Hours (Wh)."""
    today_runtime: int
    """Today runtime in minutes."""

    def to_dict(self) -> dict:
        """Gets all the properties of this result as a dictionary.

        Returns:
            dict: The result as a dictionary.
        """



from tapo.device_management_ext import DeviceManagementExt
from tapo.responses import DeviceInfoPlugResult, DeviceUsageResult

class PlugHandler(DeviceManagementExt):
    """Handler for the [P100](https://www.tapo.com/en/search/?q=P100) and
    [P105](https://www.tapo.com/en/search/?q=P105) devices.
    """

    def __init__(self, handler: object):
        """Private constructor.
        It should not be called from outside the tapo library.
        """

    async def refresh_session(self) -> None:
        """Refreshes the authentication session."""

    async def on(self) -> None:
        """Turns *on* the device."""

    async def off(self) -> None:
        """Turns *off* the device."""

    async def get_device_info(self) -> DeviceInfoPlugResult:
        """Returns *device info* as `DeviceInfoPlugResult`.
        It is not guaranteed to contain all the properties returned from the Tapo API.
        If the deserialization fails, or if a property that you care about it's not present,
        try `PlugHandler.get_device_info_json`.

        Returns:
            DeviceInfoPlugResult: Device info of Tapo P100 and P105.
            Superset of `GenericDeviceInfoResult`.
        """

    async def get_device_info_json(self) -> dict:
        """Returns *device info* as json.
        It contains all the properties returned from the Tapo API.

        Returns:
            dict: Device info as a dictionary.
        """

    async def get_device_usage(self) -> DeviceUsageResult:
        """Returns *device usage* as `DeviceUsageResult`.

        Returns:
            DeviceUsageResult: Contains the time usage.
        """



from datetime import datetime
from typing import List, Optional

class EnergyDataResult:
    """Energy data result for the requested `EnergyDataInterval`."""

    local_time: datetime
    """Local time of the device."""

    start_date_time: datetime
    """Start date and time of this result in UTC.
    This value is provided in the `get_energy_data` request and is passed through.
    Note that it may not align with the returned data if the method is used beyond its specified capabilities."""

    entries: List[EnergyDataIntervalResult]
    """List of energy data entries."""

    interval_length: int
    """Interval length in minutes."""

    def to_dict(self) -> dict:
        """Gets all the properties of this result as a dictionary.

        Returns:
            dict: The result as a dictionary.
        """

class EnergyDataIntervalResult:
    """Energy data result for a specific interval."""

    start_date_time: datetime
    """Start date and time of this interval in UTC."""

    energy: Optional[int]
    """Energy in Watt Hours (Wh)."""

    def to_dict(self) -> dict:
        """Gets all the properties of this result as a dictionary.

        Returns:
            dict: The result as a dictionary.
        """



from dataclasses import dataclass
from typing import Type, Union

from tapo import (
    ColorLightHandler,
    GenericDeviceHandler,
    HubHandler,
    LightHandler,
    PlugEnergyMonitoringHandler,
    PlugHandler,
    PowerStripEnergyMonitoringHandler,
    PowerStripHandler,
    RgbicLightStripHandler,
    RgbLightStripHandler,
)
from tapo.responses import (
    DeviceInfoColorLightResult,
    DeviceInfoGenericResult,
    DeviceInfoHubResult,
    DeviceInfoLightResult,
    DeviceInfoPlugEnergyMonitoringResult,
    DeviceInfoPlugResult,
    DeviceInfoPowerStripResult,
    DeviceInfoRgbicLightStripResult,
    DeviceInfoRgbLightStripResult,
)

class MaybeDiscoveryResult:
    """Potential result of the device discovery process. Using `get` will return the actual result or raise an exception."""

    def get(
        self,
    ) -> Union[
        GenericDevice,
        Light,
        ColorLight,
        RgbLightStrip,
        RgbicLightStrip,
        Plug,
        PlugEnergyMonitoring,
        PowerStrip,
        PowerStripEnergyMonitoring,
        Hub,
    ]:
        """Retrieves the actual discovery result or raises an exception."""

@dataclass
class GenericDevice:
    """A Generic Tapo device.

    If you believe this device is already supported, or would like to explore adding support for a currently
    unsupported model, please [open an issue on GitHub](https://github.com/mihai-dinculescu/tapo/issues)
    to start the discussion.
    """

    device_info: DeviceInfoGenericResult
    """Device info of a Generic Tapo device.
    
    If you believe this device is already supported, or would like to explore adding support for a currently
    unsupported model, please [open an issue on GitHub](https://github.com/mihai-dinculescu/tapo/issues)
    to start the discussion.
    """

    handler: GenericDeviceHandler
    """Handler for generic devices. It provides the functionality common to all Tapo [devices](https://www.tapo.com/en/).
    
    If you believe this device is already supported, or would like to explore adding support for a currently
    unsupported model, please [open an issue on GitHub](https://github.com/mihai-dinculescu/tapo/issues)
    to start the discussion.
    """

    __match_args__ = (
        "device_info",
        "handler",
    )

@dataclass
class Light:
    """Tapo L510, L520 and L610 devices."""

    device_info: DeviceInfoLightResult
    """Device info of Tapo L510, L520 and L610."""

    handler: LightHandler
    """Handler for the [L510](https://www.tapo.com/en/search/?q=L510),
    [L520](https://www.tapo.com/en/search/?q=L520) and
    [L610](https://www.tapo.com/en/search/?q=L610) devices.
    """

    __match_args__ = (
        "device_info",
        "handler",
    )

@dataclass
class ColorLight:
    """Tapo L530, L535 and L630 devices."""

    device_info: DeviceInfoColorLightResult
    """Device info of Tapo L530, L535 and L630."""

    handler: ColorLightHandler
    """Handler for the [L530](https://www.tapo.com/en/search/?q=L530),
    [L535](https://www.tapo.com/en/search/?q=L535) and
    [L630](https://www.tapo.com/en/search/?q=L630) devices.
    """

    __match_args__ = (
        "device_info",
        "handler",
    )

@dataclass
class RgbLightStrip:
    """Tapo L900 devices."""

    device_info: DeviceInfoRgbLightStripResult
    """Device info of Tapo L900."""

    handler: RgbLightStripHandler
    """Handler for the [L900](https://www.tapo.com/en/search/?q=L900) devices."""

    __match_args__ = (
        "device_info",
        "handler",
    )

@dataclass
class RgbicLightStrip:
    """Tapo L920 and L930 devices."""

    device_info: DeviceInfoRgbicLightStripResult
    """Device info of Tapo L920 and L930."""

    handler: RgbicLightStripHandler
    """Handler for the [L920](https://www.tapo.com/en/search/?q=L920) and
    [L930](https://www.tapo.com/en/search/?q=L930) devices."""

    __match_args__ = (
        "device_info",
        "handler",
    )

@dataclass
class Plug:
    """Tapo P100 and P105 devices."""

    device_info: DeviceInfoPlugResult
    """Device info of Tapo P100 and P105."""

    handler: PlugHandler
    """Handler for the [P100](https://www.tapo.com/en/search/?q=P100) and
    [P105](https://www.tapo.com/en/search/?q=P105) devices."""

    __match_args__ = (
        "device_info",
        "handler",
    )

@dataclass
class PlugEnergyMonitoring:
    """Tapo P110, P110M and P115 devices."""

    device_info: DeviceInfoPlugEnergyMonitoringResult
    """Device info of Tapo P110, P110M and P115."""

    handler: PlugEnergyMonitoringHandler
    """Handler for the [P110](https://www.tapo.com/en/search/?q=P110),
    [P110M](https://www.tapo.com/en/search/?q=P110M) and
    [P115](https://www.tapo.com/en/search/?q=P115) devices."""

    __match_args__ = (
        "device_info",
        "handler",
    )

@dataclass
class PowerStrip:
    """Tapo P300 and P306 devices."""

    device_info: DeviceInfoPowerStripResult
    """Device info of Tapo P300 and P306."""

    handler: PowerStripHandler
    """Handler for the [P300](https://www.tp-link.com/en/search/?q=P300) and
    [P306](https://www.tp-link.com/us/search/?q=P306) devices.
    """

    __match_args__ = (
        "device_info",
        "handler",
    )

@dataclass
class PowerStripEnergyMonitoring:
    """Tapo P304M and P316M devices."""

    device_info: DeviceInfoPowerStripResult
    """Device info of Tapo P304M and P316M."""

    handler: PowerStripEnergyMonitoringHandler
    """Handler for the [P304M](https://www.tp-link.com/uk/search/?q=P304M) and
    [P316M](https://www.tp-link.com/us/search/?q=P316M) devices.
    """

    __match_args__ = (
        "device_info",
        "handler",
    )

@dataclass
class Hub:
    """Tapo H100 devices."""

    device_info: DeviceInfoHubResult
    """Device info of Tapo H100."""

    handler: HubHandler
    """Handler for the [H100](https://www.tapo.com/en/search/?q=H100) devices."""

    __match_args__ = (
        "device_info",
        "handler",
    )

class DiscoveryResult:
    """Result of the device discovery process."""

    GenericDevice: Type[GenericDevice] = GenericDevice
    Light: Type[Light] = Light
    ColorLight: Type[ColorLight] = ColorLight
    RgbLightStrip: Type[RgbLightStrip] = RgbLightStrip
    RgbicLightStrip: Type[RgbicLightStrip] = RgbicLightStrip
    Plug: Type[Plug] = Plug
    PlugEnergyMonitoring: Type[PlugEnergyMonitoring] = PlugEnergyMonitoring
    PowerStrip: Type[PowerStrip] = PowerStrip
    PowerStripEnergyMonitoring: Type[PowerStripEnergyMonitoring] = PowerStripEnergyMonitoring
    Hub: Type[Hub] = Hub



"""P304M and P316M Example"""

import asyncio
from datetime import datetime, timedelta, timezone
import os

from tapo import ApiClient
from tapo.requests import EnergyDataInterval, PowerDataInterval


async def main():
    tapo_username = os.getenv("TAPO_USERNAME")
    tapo_password = os.getenv("TAPO_PASSWORD")
    ip_address = os.getenv("IP_ADDRESS")

    client = ApiClient(tapo_username, tapo_password)
    power_strip = await client.p304(ip_address)

    device_info = await power_strip.get_device_info()
    print(f"Device info: {device_info.to_dict()}")

    print("Getting child devices...")
    child_device_list = await power_strip.get_child_device_list()
    print(f"Found {len(child_device_list)} plugs")

    for index, child in enumerate(child_device_list):
        print(f"=== ({index + 1}) {child.nickname} ===")
        print(f"Device ID: {child.device_id}")
        print(f"State: {child.device_on}")

        plug = await power_strip.plug(device_id=child.device_id)

        print("Turning device on...")
        await plug.on()

        print("Waiting 2 seconds...")
        await asyncio.sleep(2)

        print("Turning device off...")
        await plug.off()

        print("Waiting 2 seconds...")
        await asyncio.sleep(2)

        current_power = await plug.get_current_power()
        print(f"Current power: {current_power.to_dict()}")

        device_usage = await plug.get_device_usage()
        print(f"Device usage: {device_usage.to_dict()}")

        energy_usage = await plug.get_energy_usage()
        print(f"Energy usage: {energy_usage.to_dict()}")

        today = datetime.now(timezone.utc)

        # Energy data - Hourly interval
        # `start_date` and `end_date` are an inclusive interval that must not be greater than 8 days.
        energy_data_hourly = await plug.get_energy_data(EnergyDataInterval.Hourly, today)
        print(
            "Energy data (hourly): "
            f"Start date time '{energy_data_hourly.start_date_time}', "
            f"Entries {len(energy_data_hourly.entries)}, "
            f"First entry: {energy_data_hourly.entries[0].to_dict() if energy_data_hourly.entries else None}"
        )

        # Energy data - Daily interval
        # `start_date` must be the first day of a quarter.
        energy_data_daily = await plug.get_energy_data(
            EnergyDataInterval.Daily,
            datetime(today.year, get_quarter_start_month(today), 1),
        )
        print(
            "Energy data (daily): "
            f"Start date time '{energy_data_daily.start_date_time}', "
            f"Entries {len(energy_data_daily.entries)}, "
            f"First entry: {energy_data_daily.entries[0].to_dict() if energy_data_daily.entries else None}"
        )

        # Energy data - Monthly interval
        # `start_date` must be the first day of a year.
        energy_data_monthly = await plug.get_energy_data(
            EnergyDataInterval.Monthly,
            datetime(today.year, 1, 1),
        )
        print(
            "Energy data (monthly): "
            f"Start date time '{energy_data_monthly.start_date_time}', "
            f"Entries {len(energy_data_monthly.entries)}, "
            f"First entry: {energy_data_monthly.entries[0].to_dict() if energy_data_monthly.entries else None}"
        )

        # Power data - Every 5 minutes interval
        # `start_date_time` and `end_date_time` describe an exclusive interval.
        # If the result would yield more than 144 entries (i.e. 12 hours),
        # the `end_date_time` will be adjusted to an earlier date and time.
        power_data_every_5_minutes = await plug.get_power_data(
            PowerDataInterval.Every5Minutes,
            today - timedelta(hours=12),
            today,
        )
        print(
            "Power data (every 5 minutes): "
            f"Start date time '{power_data_every_5_minutes.start_date_time}', "
            f"End date time '{power_data_every_5_minutes.end_date_time}', "
            f"Entries {len(power_data_every_5_minutes.entries)}, "
            f"First entry: {power_data_every_5_minutes.entries[0].to_dict() if power_data_every_5_minutes.entries else None}"
        )

        # Power data - Hourly interval
        # `start_date_time` and `end_date_time` describe an exclusive interval.
        # If the result would yield more than 144 entries (i.e. 6 days),
        # the `end_date_time` will be adjusted to an earlier date and time.
        power_data_hourly = await plug.get_power_data(
            PowerDataInterval.Hourly,
            today - timedelta(days=3),
            today,
        )
        print(
            "Power data (hourly): "
            f"Start date time '{power_data_hourly.start_date_time}', "
            f"End date time '{power_data_hourly.end_date_time}', "
            f"Entries {len(power_data_hourly.entries)}, "
            f"First entry: {power_data_hourly.entries[0].to_dict() if power_data_hourly.entries else None}"
        )


def get_quarter_start_month(today: datetime) -> int:
    return ((today.month - 1) // 3) * 3 + 1


if __name__ == "__main__":
    asyncio.run(main())



"""P110, P110M and P115 Example"""

import asyncio
import os
from datetime import datetime, timedelta, timezone

from tapo import ApiClient
from tapo.requests import EnergyDataInterval, PowerDataInterval


async def main():
    tapo_username = os.getenv("TAPO_USERNAME")
    tapo_password = os.getenv("TAPO_PASSWORD")
    ip_address = os.getenv("IP_ADDRESS")

    client = ApiClient(tapo_username, tapo_password)
    device = await client.p110(ip_address)

    print("Turning device on...")
    await device.on()

    print("Waiting 2 seconds...")
    await asyncio.sleep(2)

    print("Turning device off...")
    await device.off()

    device_info = await device.get_device_info()
    print(f"Device info: {device_info.to_dict()}")

    current_power = await device.get_current_power()
    print(f"Current power: {current_power.to_dict()}")

    device_usage = await device.get_device_usage()
    print(f"Device usage: {device_usage.to_dict()}")

    energy_usage = await device.get_energy_usage()
    print(f"Energy usage: {energy_usage.to_dict()}")

    today = datetime.now(timezone.utc)

    # Energy data - Hourly interval
    # `start_date` and `end_date` are an inclusive interval that must not be greater than 8 days.
    energy_data_hourly = await device.get_energy_data(EnergyDataInterval.Hourly, today)
    print(
        "Energy data (hourly): "
        f"Start date time '{energy_data_hourly.start_date_time}', "
        f"Entries {len(energy_data_hourly.entries)}, "
        f"First entry: {energy_data_hourly.entries[0].to_dict() if energy_data_hourly.entries else None}"
    )

    # Energy data - Daily interval
    # `start_date` must be the first day of a quarter.
    energy_data_daily = await device.get_energy_data(
        EnergyDataInterval.Daily,
        datetime(today.year, get_quarter_start_month(today), 1),
    )
    print(
        "Energy data (daily): "
        f"Start date time '{energy_data_daily.start_date_time}', "
        f"Entries {len(energy_data_daily.entries)}, "
        f"First entry: {energy_data_daily.entries[0].to_dict() if energy_data_daily.entries else None}"
    )

    # Energy data - Monthly interval
    # `start_date` must be the first day of a year.
    energy_data_monthly = await device.get_energy_data(
        EnergyDataInterval.Monthly,
        datetime(today.year, 1, 1),
    )
    print(
        "Energy data (monthly): "
        f"Start date time '{energy_data_monthly.start_date_time}', "
        f"Entries {len(energy_data_monthly.entries)}, "
        f"First entry: {energy_data_monthly.entries[0].to_dict() if energy_data_monthly.entries else None}"
    )

    # Power data - Every 5 minutes interval
    # `start_date_time` and `end_date_time` describe an exclusive interval.
    # If the result would yield more than 144 entries (i.e. 12 hours),
    # the `end_date_time` will be adjusted to an earlier date and time.
    power_data_every_5_minutes = await device.get_power_data(
        PowerDataInterval.Every5Minutes,
        today - timedelta(hours=12),
        today,
    )
    print(
        "Power data (every 5 minutes): "
        f"Start date time '{power_data_every_5_minutes.start_date_time}', "
        f"End date time '{power_data_every_5_minutes.end_date_time}', "
        f"Entries {len(power_data_every_5_minutes.entries)}, "
        f"First entry: {power_data_every_5_minutes.entries[0].to_dict() if power_data_every_5_minutes.entries else None}"
    )

    # Power data - Hourly interval
    # `start_date_time` and `end_date_time` describe an exclusive interval.
    # If the result would yield more than 144 entries (i.e. 6 days),
    # the `end_date_time` will be adjusted to an earlier date and time.
    power_data_hourly = await device.get_power_data(
        PowerDataInterval.Hourly,
        today - timedelta(days=3),
        today,
    )
    print(
        "Power data (hourly): "
        f"Start date time '{power_data_hourly.start_date_time}', "
        f"End date time '{power_data_hourly.end_date_time}', "
        f"Entries {len(power_data_hourly.entries)}, "
        f"First entry: {power_data_hourly.entries[0].to_dict() if power_data_hourly.entries else None}"
    )


def get_quarter_start_month(today: datetime) -> int:
    return ((today.month - 1) // 3) * 3 + 1


if __name__ == "__main__":
    asyncio.run(main())



from tapo.device_management_ext import DeviceManagementExt
from tapo.responses import DeviceInfoLightResult, DeviceUsageResult

class LightHandler(DeviceManagementExt):
    """Handler for the [L510](https://www.tapo.com/en/search/?q=L510),
    [L520](https://www.tapo.com/en/search/?q=L520) and
    [L610](https://www.tapo.com/en/search/?q=L610) devices."""

    def __init__(self, handler: object):
        """Private constructor.
        It should not be called from outside the tapo library.
        """

    async def refresh_session(self) -> None:
        """Refreshes the authentication session."""

    async def on(self) -> None:
        """Turns *on* the device."""

    async def off(self) -> None:
        """Turns *off* the device."""

    async def get_device_info(self) -> DeviceInfoLightResult:
        """Returns *device info* as `DeviceInfoLightResult`.
        It is not guaranteed to contain all the properties returned from the Tapo API.
        If the deserialization fails, or if a property that you care about it's not present,
        try `LightHandler.get_device_info_json`.

        Returns:
            DeviceInfoLightResult: Device info of Tapo L510, L520 and L610.
            Superset of `GenericDeviceInfoResult`.
        """

    async def get_device_info_json(self) -> dict:
        """Returns *device info* as json.
        It contains all the properties returned from the Tapo API.

        Returns:
            dict: Device info as a dictionary.
        """

    async def get_device_usage(self) -> DeviceUsageResult:
        """Returns *device usage* as `DeviceUsageResult`.

        Returns:
            DeviceUsageResult: Contains the time usage.
        """

    async def set_brightness(self, brightness: int) -> None:
        """Sets the *brightness* and turns *on* the device.

        Args:
            brightness (int): between 1 and 100
        """



from datetime import datetime

from tapo.device_management_ext import DeviceManagementExt
from tapo.requests import EnergyDataInterval, PowerDataInterval
from tapo.responses import (
    CurrentPowerResult,
    DeviceInfoPlugEnergyMonitoringResult,
    DeviceUsageEnergyMonitoringResult,
    EnergyDataResult,
    EnergyUsageResult,
    PowerDataResult,
)

class PlugEnergyMonitoringHandler(DeviceManagementExt):
    """Handler for the [P110](https://www.tapo.com/en/search/?q=P110),
    [P110M](https://www.tapo.com/en/search/?q=P110M) and
    [P115](https://www.tapo.com/en/search/?q=P115) devices.
    """

    def __init__(self, handler: object):
        """Private constructor.
        It should not be called from outside the tapo library.
        """

    async def refresh_session(self) -> None:
        """Refreshes the authentication session."""

    async def on(self) -> None:
        """Turns *on* the device."""

    async def off(self) -> None:
        """Turns *off* the device."""

    async def get_device_info(self) -> DeviceInfoPlugEnergyMonitoringResult:
        """Returns *device info* as `DeviceInfoPlugEnergyMonitoringResult`.
        It is not guaranteed to contain all the properties returned from the Tapo API.
        If the deserialization fails, or if a property that you care about it's not present,
        try `PlugEnergyMonitoringHandler.get_device_info_json`.

        Returns:
            DeviceInfoPlugEnergyMonitoringResult: Device info of P110, P110M and P115.
            Superset of `GenericDeviceInfoResult`.
        """

    async def get_device_info_json(self) -> dict:
        """Returns *device info* as json.
        It contains all the properties returned from the Tapo API.

        Returns:
            dict: Device info as a dictionary.
        """

    async def get_current_power(self) -> CurrentPowerResult:
        """Returns *current power* as `CurrentPowerResult`.

        Returns:
            CurrentPowerResult: Contains the current power reading of the device.
        """

    async def get_device_usage(self) -> DeviceUsageEnergyMonitoringResult:
        """Returns *device usage* as `DeviceUsageResult`.

        Returns:
            DeviceUsageEnergyMonitoringResult:
            Contains the time usage, the power consumption, and the energy savings of the device.
        """

    async def get_energy_usage(self) -> EnergyUsageResult:
        """Returns *energy usage* as `EnergyUsageResult`.

        Returns:
            EnergyUsageResult:
            Contains local time, current power and the energy usage and runtime for today and for the current month.
        """

    async def get_energy_data(
        self,
        interval: EnergyDataInterval,
        start_date: datetime,
        end_date: datetime = None,
    ) -> EnergyDataResult:
        """Returns *energy data* as `EnergyDataResult`.

        Returns:
            EnergyDataResult: Energy data result for the requested `EnergyDataInterval`.
        """

    async def get_power_data(
        self,
        interval: PowerDataInterval,
        start_date_time: datetime,
        end_date_time: datetime,
    ) -> PowerDataResult:
        """Returns *power data* as `PowerDataResult`.

        Returns:
            PowerDataResult: Power data result for the requested `PowerDataInterval`.
        """



from datetime import datetime
from typing import List, Optional

class PowerDataResult:
    """Power data result for the requested `PowerDataInterval`."""

    start_date_time: datetime
    """Start date and time of this result in UTC."""

    end_date_time: datetime
    """End date and time of this result in UTC."""

    entries: List[PowerDataIntervalResult]
    """List of power data entries."""

    interval_length: int
    """Interval length in minutes."""

    def to_dict(self) -> dict:
        """Gets all the properties of this result as a dictionary.

        Returns:
            dict: The result as a dictionary.
        """

class PowerDataIntervalResult:
    """Power data result for a specific interval."""

    start_date_time: datetime
    """Start date and time of this interval in UTC."""

    power: Optional[int]
    """Power in Watts (W). `None` if no data is available for this interval."""

    def to_dict(self) -> dict:
        """Gets all the properties of this result as a dictionary.

        Returns:
            dict: The result as a dictionary.
        """



from __future__ import annotations
from typing import AsyncIterable, Iterable, Iterator, AsyncIterator

from .discovery_result import MaybeDiscoveryResult

class DeviceDiscoveryIter(Iterator[MaybeDiscoveryResult], AsyncIterator[MaybeDiscoveryResult]):
    def __iter__(self) -> DeviceDiscoveryIter: ...
    def __aiter__(self) -> DeviceDiscoveryIter: ...
    def __next__(self) -> MaybeDiscoveryResult: ...
    async def __anext__(self) -> MaybeDiscoveryResult: ...

class DeviceDiscovery(Iterable[MaybeDiscoveryResult], AsyncIterable[MaybeDiscoveryResult]):
    def __iter__(self) -> DeviceDiscoveryIter: ...
    def __aiter__(self) -> DeviceDiscoveryIter: ...



from typing import Type
from dataclasses import dataclass

@dataclass
class LastStates:
    __match_args__: tuple[str, ...] = ()

@dataclass
class Custom:
    state: PlugState

    __match_args__ = ("state",)

class DefaultPlugState:
    """Plug Default State."""

    LastStates: Type[LastStates] = LastStates
    Custom: Type[Custom] = Custom

class PlugState:
    """Plug State."""

    on: bool

    def to_dict(self) -> dict:
        """Gets all the properties of this result as a dictionary.

        Returns:
            dict: The result as a dictionary.
        """
